//---------------------------------------------------------------------------------------------------------------------------------
// Module/file name: IO48XPTestDlg.cpp
// Module Description: IO48XPTestDlg class implementation file
// Creation date: 11/17/07
// Author/s: George Hennessee
// Executable program name: IO48XPTest.exe
// Executable program type: Windows 2K/XP Application
//
// NOTE: This source code is written assuming a 132-character line width. For decent readability select font, page width, page
//       orientation, etc. to yield a 132-character unbroken line on your word processor screen. Be aware that any saving of this
//       source code file must be in TEXT ONLY format (no embedded codes) to avoid choking the C compiler. - GH
// Revision History:
//    Date      Release Rev    Author    Description
//  --------    -----------    ------    --------------------------------------------------------------------------------
//  11/19/07       1.00          GH      Original release
//
//              ******************************** COPYRIGHT NOTICE AND DISCLAIMER ********************************
//   This source file is Copyright 2007 by WinSystems, Inc., Arlington, Texas. All rights reserved. This file is provided 'as-is'
//   and WinSystems makes no warranty as to usability of this code. Disclosure or use of this code except as provided below is
//   strictly prohibited. Refer to the WinSystems Hardware/Software Sales Agreement for other terms and conditions that may apply.
//---------------------------------------------------------------------------------------------------------------------------------

#include "stdafx.h"
#include "IO48XPTest.h"
#include "IO48XPTestDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#ifdef SOFTWARE_DEBUG
#pragma warning (disable : 4101) // disable warning about 'unreferenced local variable'
#endif


/////////////////////////////////////////////////////////////////////////////
// CAboutDlg dialog used for App About

class CAboutDlg : public CDialog
{
public:
	CAboutDlg();

// Dialog Data
	//{{AFX_DATA(CAboutDlg)
	enum { IDD = IDD_ABOUTBOX };
	//}}AFX_DATA

	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CAboutDlg)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
protected:
	//{{AFX_MSG(CAboutDlg)
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
	//{{AFX_DATA_INIT(CAboutDlg)
	//}}AFX_DATA_INIT
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAboutDlg)
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
	//{{AFX_MSG_MAP(CAboutDlg)
		// No message handlers
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CIO48XPTestDlg dialog

CIO48XPTestDlg::CIO48XPTestDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CIO48XPTestDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CIO48XPTestDlg)
	m_byAR1Value = 0;
	m_byAR2Value = 0;
	m_byAW1Value = 0;
	m_byAW2Value = 0;
	m_byBR1Value = 0;
	m_byBR2Value = 0;
	m_byBW1Value = 0;
	m_byBW2Value = 0;
	m_nDevice = -1;
	m_nModeA1 = -1;
	m_nModeB1 = -1;
	m_byCR1LOValue = 0;
	m_byCW1LOValue = 0;
	m_nModeC1LO = -1;
	m_byCR1HIValue = 0;
	m_byCW1HIValue = 0;
	m_nModeC1HI = -1;
	m_byCR2HIValue = 0;
	m_byCR2LOValue = 0;
	m_byCW2HIValue = 0;
	m_byCW2LOValue = 0;
	m_nModeA2 = -1;
	m_nModeB2 = -1;
	m_nModeC2HI = -1;
	m_nModeC2LO = -1;
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);

	m_hDeviceThread = INVALID_HANDLE_VALUE;
	m_ThreadData.hWIOHandle = INVALID_HANDLE_VALUE;
	m_ThreadData.hExitThread = INVALID_HANDLE_VALUE;
	m_ThreadData.hThreadExited = INVALID_HANDLE_VALUE;
	m_ThreadData.fExitThread = FALSE;
	m_ThreadData.hwndDialog = NULL;
}

// ****************************************************************************
//
//	DeviceThread
//
//	Parameters:
//
//		PVOID		pContext
//					[IN] Pointer to a THREAD_DATA structure.
//
//	Remarks:	Thread to wait for interrupt events from the driver.
//
//	Returns:	0
//
/*static*/
DWORD WINAPI CIO48XPTestDlg::DeviceThread(PVOID pContext)
{
	PTHREAD_DATA	pThreadData =
		reinterpret_cast<PTHREAD_DATA>(pContext);

	// Continue with thread until told to exit.
	while (FALSE == pThreadData->fExitThread)
	{
		// The only reason this code is here is because the
		// test application allows for the use of the driver
		// without interrupts configured.  If interrupts
		// were preconfigured prior to the execution of this
		// thread, we could simply issue the DeviceIoControl
		// and then call WaitForMultipleObjects to block
		// until the thread is signaled to shutdown or an
		// interrupt occurred in the driver.

		LeaveCriticalSection(&(pThreadData->hThreadCS));
	}

	// Signal thread is exiting.
	SetEvent(pThreadData->hThreadExited);

	return 0;
}


void CIO48XPTestDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CIO48XPTestDlg)
	DDX_Control(pDX, IDC_EDITCW2LO, m_CW2LOEditCtrl);
	DDX_Control(pDX, IDC_EDITCW2HI, m_CW2HIEditCtrl);
	DDX_Control(pDX, IDC_EDITCR2LO, m_CR2LOEditCtrl);
	DDX_Control(pDX, IDC_EDITCR2HI, m_CR2HIEditCtrl);
	DDX_Control(pDX, IDC_BUTTONCW2LO, m_ctlButtonCW2LO);
	DDX_Control(pDX, IDC_BUTTONCW2HI, m_ctlButtonCW2HI);
	DDX_Control(pDX, IDC_BUTTONCR2LO, m_ctlButtonCR2LO);
	DDX_Control(pDX, IDC_BUTTONCR2HI, m_ctlButtonCR2HI);
	DDX_Control(pDX, IDC_EDITCW1HI, m_CW1HIEditCtrl);
	DDX_Control(pDX, IDC_EDITCR1HI, m_CR1HIEditCtrl);
	DDX_Control(pDX, IDC_BUTTONCW1HI, m_ctlButtonCW1HI);
	DDX_Control(pDX, IDC_BUTTONCR1HI, m_ctlButtonCR1HI);
	DDX_Control(pDX, IDC_EDITCW1LO, m_CW1LOEditCtrl);
	DDX_Control(pDX, IDC_EDITCR1LO, m_CR1LOEditCtrl);
	DDX_Control(pDX, IDC_BUTTONCW1LO, m_ctlButtonCW1LO);
	DDX_Control(pDX, IDC_BUTTONCR1LO, m_ctlButtonCR1LO);
	DDX_Control(pDX, IDC_BUTTONBW2, m_ctlButtonBW2);
	DDX_Control(pDX, IDC_BUTTONBW1, m_ctlButtonBW1);
	DDX_Control(pDX, IDC_BUTTONBR2, m_ctlButtonBR2);
	DDX_Control(pDX, IDC_BUTTONBR1, m_ctlButtonBR1);
	DDX_Control(pDX, IDC_BUTTONAW2, m_ctlButtonAW2);
	DDX_Control(pDX, IDC_BUTTONAW1, m_ctlButtonAW1);
	DDX_Control(pDX, IDC_BUTTONAR2, m_ctlButtonAR2);
	DDX_Control(pDX, IDC_BUTTONAR1, m_ctlButtonAR1);
	DDX_Control(pDX, IDC_BUTTONCloseDriver, m_ctlButtonCloseDevice);
	DDX_Control(pDX, IDC_BUTTONOpenDriver, m_ctlButtonOpenDevice);
	DDX_Control(pDX, IDC_COMBODeviceName, m_ctlDevice);
	DDX_Control(pDX, IDC_EDITBW2, m_BW2EditCtrl);
	DDX_Control(pDX, IDC_EDITBW1, m_BW1EditCtrl);
	DDX_Control(pDX, IDC_EDITBR2, m_BR2EditCtrl);
	DDX_Control(pDX, IDC_EDITBR1, m_BR1EditCtrl);
	DDX_Control(pDX, IDC_EDITAW2, m_AW2EditCtrl);
	DDX_Control(pDX, IDC_EDITAW1, m_AW1EditCtrl);
	DDX_Control(pDX, IDC_EDITAR2, m_AR2EditCtrl);
	DDX_Control(pDX, IDC_EDITAR1, m_AR1EditCtrl);
	DDX_Text(pDX, IDC_EDITAR1, m_byAR1Value);
	DDV_MinMaxByte(pDX, m_byAR1Value, 0, 255);
	DDX_Text(pDX, IDC_EDITAR2, m_byAR2Value);
	DDV_MinMaxByte(pDX, m_byAR2Value, 0, 255);
	DDX_Text(pDX, IDC_EDITAW1, m_byAW1Value);
	DDV_MinMaxByte(pDX, m_byAW1Value, 0, 255);
	DDX_Text(pDX, IDC_EDITAW2, m_byAW2Value);
	DDV_MinMaxByte(pDX, m_byAW2Value, 0, 255);
	DDX_Text(pDX, IDC_EDITBR1, m_byBR1Value);
	DDV_MinMaxByte(pDX, m_byBR1Value, 0, 255);
	DDX_Text(pDX, IDC_EDITBR2, m_byBR2Value);
	DDV_MinMaxByte(pDX, m_byBR2Value, 0, 255);
	DDX_Text(pDX, IDC_EDITBW1, m_byBW1Value);
	DDV_MinMaxByte(pDX, m_byBW1Value, 0, 255);
	DDX_Text(pDX, IDC_EDITBW2, m_byBW2Value);
	DDV_MinMaxByte(pDX, m_byBW2Value, 0, 255);
	DDX_CBIndex(pDX, IDC_COMBODeviceName, m_nDevice);
	DDX_Radio(pDX, IDC_RADIOAW1, m_nModeA1);
	DDX_Radio(pDX, IDC_RADIOBW1, m_nModeB1);
	DDX_Text(pDX, IDC_EDITCR1LO, m_byCR1LOValue);
	DDV_MinMaxByte(pDX, m_byCR1LOValue, 0, 15);
	DDX_Text(pDX, IDC_EDITCW1LO, m_byCW1LOValue);
	DDV_MinMaxByte(pDX, m_byCW1LOValue, 0, 15);
	DDX_Radio(pDX, IDC_RADIOCW1LO, m_nModeC1LO);
	DDX_Text(pDX, IDC_EDITCR1HI, m_byCR1HIValue);
	DDV_MinMaxByte(pDX, m_byCR1HIValue, 0, 15);
	DDX_Text(pDX, IDC_EDITCW1HI, m_byCW1HIValue);
	DDV_MinMaxByte(pDX, m_byCW1HIValue, 0, 15);
	DDX_Radio(pDX, IDC_RADIOCW1HI, m_nModeC1HI);
	DDX_Text(pDX, IDC_EDITCR2HI, m_byCR2HIValue);
	DDV_MinMaxByte(pDX, m_byCR2HIValue, 0, 15);
	DDX_Text(pDX, IDC_EDITCR2LO, m_byCR2LOValue);
	DDV_MinMaxByte(pDX, m_byCR2LOValue, 0, 15);
	DDX_Text(pDX, IDC_EDITCW2HI, m_byCW2HIValue);
	DDV_MinMaxByte(pDX, m_byCW2HIValue, 0, 15);
	DDX_Text(pDX, IDC_EDITCW2LO, m_byCW2LOValue);
	DDV_MinMaxByte(pDX, m_byCW2LOValue, 0, 15);
	DDX_Radio(pDX, IDC_RADIOAW2, m_nModeA2);
	DDX_Radio(pDX, IDC_RADIOBW2, m_nModeB2);
	DDX_Radio(pDX, IDC_RADIOCW2HI, m_nModeC2HI);
	DDX_Radio(pDX, IDC_RADIOCW2LO, m_nModeC2LO);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CIO48XPTestDlg, CDialog)
	//{{AFX_MSG_MAP(CIO48XPTestDlg)
	ON_WM_SYSCOMMAND()
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDC_BUTTONCloseDriver, OnBUTTONCloseDriver)
	ON_BN_CLICKED(IDC_BUTTONOpenDriver, OnBUTTONOpenDriver)
	ON_WM_DESTROY()
	ON_BN_CLICKED(IDC_BUTTONAR1, OnButtonar1)
	ON_BN_CLICKED(IDC_BUTTONAR2, OnButtonar2)
	ON_BN_CLICKED(IDC_BUTTONBR1, OnButtonbr1)
	ON_BN_CLICKED(IDC_BUTTONBR2, OnButtonbr2)
	ON_BN_CLICKED(IDC_BUTTONBW1, OnButtonbw1)
	ON_BN_CLICKED(IDC_BUTTONBW2, OnButtonbw2)
	ON_BN_CLICKED(IDC_BUTTONAW1, OnButtonaw1)
	ON_BN_CLICKED(IDC_BUTTONAW2, OnButtonaw2)
	ON_BN_CLICKED(IDC_OK, OnClickedOk)
	ON_CBN_SELCHANGE(IDC_COMBODeviceName, OnSelchangeCOMBODeviceName)
	ON_BN_CLICKED(IDC_RADIOAW1, OnRadioaw1)
	ON_BN_CLICKED(IDC_RADIOAR1, OnRadioar1)
	ON_BN_CLICKED(IDC_RADIOBR1, OnRadiobr1)
	ON_BN_CLICKED(IDC_RADIOBW1, OnRadiobw1)
	ON_BN_CLICKED(IDC_BUTTONCR1LO, OnButtoncr1lo)
	ON_BN_CLICKED(IDC_BUTTONCW1LO, OnButtoncw1lo)
	ON_BN_CLICKED(IDC_RADIOCR1LO, OnRadiocr1lo)
	ON_BN_CLICKED(IDC_RADIOCW1LO, OnRadiocw1lo)
	ON_BN_CLICKED(IDC_RADIOCR1HI, OnRadiocr1hi)
	ON_BN_CLICKED(IDC_RADIOCW1HI, OnRadiocw1hi)
	ON_BN_CLICKED(IDC_BUTTONCW1HI, OnButtoncw1hi)
	ON_BN_CLICKED(IDC_BUTTONCR1HI, OnButtoncr1hi)
	ON_BN_CLICKED(IDC_BUTTONCR2HI, OnButtoncr2hi)
	ON_BN_CLICKED(IDC_BUTTONCR2LO, OnButtoncr2lo)
	ON_BN_CLICKED(IDC_BUTTONCW2HI, OnButtoncw2hi)
	ON_BN_CLICKED(IDC_BUTTONCW2LO, OnButtoncw2lo)
	ON_BN_CLICKED(IDC_RADIOAR2, OnRadioar2)
	ON_BN_CLICKED(IDC_RADIOAW2, OnRadioaw2)
	ON_BN_CLICKED(IDC_RADIOBR2, OnRadiobr2)
	ON_BN_CLICKED(IDC_RADIOBW2, OnRadiobw2)
	ON_BN_CLICKED(IDC_RADIOCR2HI, OnRadiocr2hi)
	ON_BN_CLICKED(IDC_RADIOCR2LO, OnRadiocr2lo)
	ON_BN_CLICKED(IDC_RADIOCW2HI, OnRadiocw2hi)
	ON_BN_CLICKED(IDC_RADIOCW2LO, OnRadiocw2lo)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CIO48XPTestDlg message handlers

BOOL CIO48XPTestDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Add "About..." menu item to system menu.

	// IDM_ABOUTBOX must be in the system command range.
	ASSERT((IDM_ABOUTBOX & 0xFFF0) == IDM_ABOUTBOX);
	ASSERT(IDM_ABOUTBOX < 0xF000);

	CMenu* pSysMenu = GetSystemMenu(FALSE);
	if (pSysMenu != NULL)
	{
		CString strAboutMenu;
		strAboutMenu.LoadString(IDS_ABOUTBOX);
		if (!strAboutMenu.IsEmpty())
		{
			pSysMenu->AppendMenu(MF_SEPARATOR);
			pSysMenu->AppendMenu(MF_STRING, IDM_ABOUTBOX, strAboutMenu);
		}
	}

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon
	
	// TODO: Add extra initialization here
	CenterWindow(GetDesktopWindow());	// center to the hpc screen

	deviceopen = FALSE;
	m_nModeA1 = 0;
	m_nModeB1 = 0;
	m_nModeC1LO = 0;
	m_nModeC1HI = 0;
	m_nModeA2 = 0;
	m_nModeB2 = 0;
	m_nModeC2LO = 0;
	m_nModeC2HI = 0;

	m_ctlDevice.SetCurSel(0);
    m_nDevice = 0;
	byU1cmdvalue = 0;
	byU2cmdvalue = 0;
	byC1value = 0;
	byC2value = 0;

	UpdateData(FALSE);      // variable data to dialog now

	// Disable controls not valid.
	UpdateDlgControls(FALSE);

	// Initialize critical section.
	InitializeCriticalSection(&m_ThreadData.hThreadCS);
	
	GetDlgItem(IDC_BUTTONOpenDriver)->SetFocus();

	return FALSE;  // return TRUE unless you set the focus to a control
	
//	return TRUE;  // return TRUE  unless you set the focus to a control
}

void CIO48XPTestDlg::OnSysCommand(UINT nID, LPARAM lParam)
{
	if ((nID & 0xFFF0) == IDM_ABOUTBOX)
	{
		CAboutDlg dlgAbout;
		dlgAbout.DoModal();
	}
	else
	{
		CDialog::OnSysCommand(nID, lParam);
	}
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CIO48XPTestDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CIO48XPTestDlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}

void CIO48XPTestDlg::UpdateDlgControls(BOOL bena)
{
    if (m_nDevice == 0 || m_nDevice == 1)
    {
	    m_ctlButtonAR1.EnableWindow(bena);
		m_ctlButtonBR1.EnableWindow(bena);
	    m_ctlButtonCR1LO.EnableWindow(bena);
	    m_ctlButtonCR1HI.EnableWindow(bena);
	    m_ctlButtonAW1.EnableWindow(bena);
	    m_ctlButtonBW1.EnableWindow(bena);
	    m_ctlButtonCW1LO.EnableWindow(bena);
	    m_ctlButtonCW1HI.EnableWindow(bena);
		GetDlgItem(IDC_RADIOAW1)->EnableWindow(bena);
		GetDlgItem(IDC_RADIOBW1)->EnableWindow(bena);
		GetDlgItem(IDC_RADIOCW1LO)->EnableWindow(bena);
		GetDlgItem(IDC_RADIOCW1HI)->EnableWindow(bena);
		GetDlgItem(IDC_RADIOAR1)->EnableWindow(bena);
		GetDlgItem(IDC_RADIOBR1)->EnableWindow(bena);
		GetDlgItem(IDC_RADIOCR1LO)->EnableWindow(bena);
		GetDlgItem(IDC_RADIOCR1HI)->EnableWindow(bena);
	    m_AR1EditCtrl.EnableWindow(bena);
	    m_BR1EditCtrl.EnableWindow(bena);
	    m_CR1LOEditCtrl.EnableWindow(bena);
	    m_CR1HIEditCtrl.EnableWindow(bena);
	    m_AW1EditCtrl.EnableWindow(bena);
	    m_BW1EditCtrl.EnableWindow(bena);
	    m_CW1LOEditCtrl.EnableWindow(bena);
	    m_CW1HIEditCtrl.EnableWindow(bena);
    }
    if (m_nDevice == 0 || m_nDevice == 2)
    {
	    m_ctlButtonAR2.EnableWindow(bena);
		m_ctlButtonBR2.EnableWindow(bena);
	    m_ctlButtonCR2LO.EnableWindow(bena);
	    m_ctlButtonCR2HI.EnableWindow(bena);
		GetDlgItem(IDC_RADIOAW2)->EnableWindow(bena);
		GetDlgItem(IDC_RADIOBW2)->EnableWindow(bena);
		GetDlgItem(IDC_RADIOCW2LO)->EnableWindow(bena);
		GetDlgItem(IDC_RADIOCW2HI)->EnableWindow(bena);
		GetDlgItem(IDC_RADIOAR2)->EnableWindow(bena);
		GetDlgItem(IDC_RADIOBR2)->EnableWindow(bena);
		GetDlgItem(IDC_RADIOCR2LO)->EnableWindow(bena);
		GetDlgItem(IDC_RADIOCR2HI)->EnableWindow(bena);
	    m_ctlButtonAW2.EnableWindow(bena);
	    m_ctlButtonBW2.EnableWindow(bena);
	    m_ctlButtonCW2LO.EnableWindow(bena);
	    m_ctlButtonCW2HI.EnableWindow(bena);
	    m_AR2EditCtrl.EnableWindow(bena);
	    m_BR2EditCtrl.EnableWindow(bena);
	    m_CR2LOEditCtrl.EnableWindow(bena);
	    m_CR2HIEditCtrl.EnableWindow(bena);
	    m_AW2EditCtrl.EnableWindow(bena);
	    m_BW2EditCtrl.EnableWindow(bena);
	    m_CW2LOEditCtrl.EnableWindow(bena);
	    m_CW2HIEditCtrl.EnableWindow(bena);
	}
}

void CIO48XPTestDlg::OnDestroy() 
{
	CDialog::OnDestroy();
	
	// Signal the thread to exit
	m_ThreadData.fExitThread = TRUE;

	// Close the device driver
	if (INVALID_HANDLE_VALUE != m_ThreadData.hWIOHandle)
	{
		CloseHandle(m_ThreadData.hWIOHandle);
		m_ThreadData.hWIOHandle = INVALID_HANDLE_VALUE;
	}

	// Now close the thread
	if (INVALID_HANDLE_VALUE != m_hDeviceThread)
	{
		// Wait for thread to exit
		WaitForSingleObject(m_ThreadData.hThreadExited, 3000);
		Sleep(10);

		CloseHandle(m_hDeviceThread);
		m_hDeviceThread = INVALID_HANDLE_VALUE;
	}

	if (INVALID_HANDLE_VALUE == m_ThreadData.hThreadExited)
	{
		CloseHandle(m_ThreadData.hThreadExited);
		m_ThreadData.hThreadExited = INVALID_HANDLE_VALUE;
	}

	DeleteCriticalSection(&m_ThreadData.hThreadCS);
}

void CIO48XPTestDlg::OnClickedOk() 
{
	CIO48XPTestDlg::OnOK();
}


void CIO48XPTestDlg::OnOK()
{
    CWnd* pwndCtrl = GetFocus();
    CWnd* pwndCtrlNext = pwndCtrl;
    int ctrl_ID = pwndCtrl->GetDlgCtrlID();

    switch (ctrl_ID)
    {
		case IDC_BUTTONOpenDriver:
			OnBUTTONOpenDriver();
			pwndCtrlNext = GetDlgItem(IDC_BUTTONCloseDriver);
			break;
		case IDC_BUTTONCloseDriver:
			OnBUTTONCloseDriver();
            pwndCtrlNext = GetDlgItem(IDC_EDITAW1);
			break;
        case IDC_EDITAW1:
            pwndCtrlNext = GetDlgItem(IDC_BUTTONAW1);
            break;
        case IDC_BUTTONAW1:
			OnButtonaw1();
            pwndCtrlNext = GetDlgItem(IDC_BUTTONAR1);
            break;
        case IDC_BUTTONAR1:
			OnButtonar1();
			pwndCtrlNext = GetDlgItem(IDC_RADIOAW1);
			break;
		case IDC_RADIOAW1:
			OnRadioaw1();
			pwndCtrlNext = GetDlgItem(IDC_RADIOAR1);
			break;
		case IDC_RADIOAR1:
			OnRadioar1();
            pwndCtrlNext = GetDlgItem(IDC_EDITBW1);
            break;
        case IDC_EDITBW1:
            pwndCtrlNext = GetDlgItem(IDC_BUTTONBW1);
            break;
        case IDC_BUTTONBW1:
            OnButtonbw1();
            pwndCtrlNext = GetDlgItem(IDC_BUTTONBR1);
            break;
        case IDC_BUTTONBR1:
            OnButtonbr1();
			pwndCtrlNext = GetDlgItem(IDC_RADIOBW1);
			break;
		case IDC_RADIOBW1:
			OnRadiobw1();
			pwndCtrlNext = GetDlgItem(IDC_RADIOBR1);
			break;
		case IDC_RADIOBR1:
			OnRadiobr1();
            pwndCtrlNext = GetDlgItem(IDC_EDITCW1LO);
            break;
        case IDC_EDITCW1LO:
            pwndCtrlNext = GetDlgItem(IDC_BUTTONCW1LO);
            break;
        case IDC_BUTTONCW1LO:
			OnButtoncw1lo();
            pwndCtrlNext = GetDlgItem(IDC_BUTTONCR1LO);
            break;
        case IDC_BUTTONCR1LO:
			OnButtoncr1lo();
			pwndCtrlNext = GetDlgItem(IDC_RADIOCW1LO);
			break;
		case IDC_RADIOCW1LO:
			OnRadiocw1lo();
			pwndCtrlNext = GetDlgItem(IDC_RADIOCR1LO);
			break;
		case IDC_RADIOCR1LO:
			OnRadiocr1lo();
            pwndCtrlNext = GetDlgItem(IDC_EDITCW1HI);
            break;
        case IDC_EDITCW1HI:
            pwndCtrlNext = GetDlgItem(IDC_BUTTONCW1HI);
            break;
        case IDC_BUTTONCW1HI:
			OnButtoncw1hi();
            pwndCtrlNext = GetDlgItem(IDC_BUTTONCR1HI);
            break;
        case IDC_BUTTONCR1HI:
            OnButtoncr1hi();
			pwndCtrlNext = GetDlgItem(IDC_RADIOCW1HI);
			break;
		case IDC_RADIOCW1HI:
			OnRadiocw1hi();
			pwndCtrlNext = GetDlgItem(IDC_RADIOCR1HI);
			break;
		case IDC_RADIOCR1HI:
			OnRadiocr1hi();
			pwndCtrlNext = GetDlgItem(IDC_EDITAW2);

			break;
        case IDC_EDITAW2:
            pwndCtrlNext = GetDlgItem(IDC_BUTTONAW2);
            break;
        case IDC_BUTTONAW2:
			OnButtonaw2();
            pwndCtrlNext = GetDlgItem(IDC_BUTTONAR2);
            break;
        case IDC_BUTTONAR2:
			OnButtonar2();
			pwndCtrlNext = GetDlgItem(IDC_RADIOAW2);
			break;
		case IDC_RADIOAW2:
			OnRadioaw2();
			pwndCtrlNext = GetDlgItem(IDC_RADIOAR2);
			break;
		case IDC_RADIOAR2:
			OnRadioar2();
            pwndCtrlNext = GetDlgItem(IDC_EDITBW2);
            break;
        case IDC_EDITBW2:
            pwndCtrlNext = GetDlgItem(IDC_BUTTONBW2);
            break;
        case IDC_BUTTONBW2:
            OnButtonbw2();
            pwndCtrlNext = GetDlgItem(IDC_BUTTONBR2);
            break;
        case IDC_BUTTONBR2:
            OnButtonbr2();
			pwndCtrlNext = GetDlgItem(IDC_RADIOBW2);
			break;
		case IDC_RADIOBW2:
			OnRadiobw2();
			pwndCtrlNext = GetDlgItem(IDC_RADIOBR2);
			break;
		case IDC_RADIOBR2:
			OnRadiobr2();
            pwndCtrlNext = GetDlgItem(IDC_EDITCW2LO);
            break;
        case IDC_EDITCW2LO:
            pwndCtrlNext = GetDlgItem(IDC_BUTTONCW2LO);
            break;
        case IDC_BUTTONCW2LO:
			OnButtoncw2lo();
            pwndCtrlNext = GetDlgItem(IDC_BUTTONCR2LO);
            break;
        case IDC_BUTTONCR2LO:
			OnButtoncr2lo();
			pwndCtrlNext = GetDlgItem(IDC_RADIOCW2LO);
			break;
		case IDC_RADIOCW2LO:
			OnRadiocw2lo();
			pwndCtrlNext = GetDlgItem(IDC_RADIOCR2LO);
			break;
		case IDC_RADIOCR2LO:
			OnRadiocr2lo();
            pwndCtrlNext = GetDlgItem(IDC_EDITCW2HI);
            break;
        case IDC_EDITCW2HI:
            pwndCtrlNext = GetDlgItem(IDC_BUTTONCW2HI);
            break;
        case IDC_BUTTONCW2HI:
			OnButtoncw2hi();
            pwndCtrlNext = GetDlgItem(IDC_BUTTONCR2HI);
            break;
        case IDC_BUTTONCR2HI:
            OnButtoncr2hi();
			pwndCtrlNext = GetDlgItem(IDC_RADIOCW2HI);
			break;
		case IDC_RADIOCW2HI:
			OnRadiocw2hi();
			pwndCtrlNext = GetDlgItem(IDC_RADIOCR2HI);
			break;
		case IDC_RADIOCR2HI:
			OnRadiocr2hi();
			pwndCtrlNext = GetDlgItem(IDC_BUTTONOpenDriver);
			break;
        case IDC_OK:
            CDialog::OnOK();
            break;
        default:
            break;
    }
    pwndCtrlNext->SetFocus();

	UpdateDlgControls(deviceopen);
}


void CIO48XPTestDlg::OnSelchangeCOMBODeviceName() 
{
	m_nDevice = m_ctlDevice.GetCurSel();
}


void CIO48XPTestDlg::OnBUTTONOpenDriver() 
{
    CWnd*       pwndCtrlNext;
	CString		strUserDeviceName = "IO48XP_0";
	CString		strDeviceName;
	IO_STRUC	IOWriteValue;
	DWORD		dwActualOut;

	strDeviceName = _T("\\\\.\\") + strUserDeviceName;

#ifndef SOFTWARE_DEBUG
	m_ThreadData.hWIOHandle = CreateFile(strDeviceName,
										 GENERIC_READ | GENERIC_WRITE,
										 0,
										 NULL,
										 OPEN_EXISTING,
										 FILE_FLAG_OVERLAPPED,
										 NULL);
	if (INVALID_HANDLE_VALUE == m_ThreadData.hWIOHandle)
	{
		CString	strErrorMsg;

		strErrorMsg.Format(_T("Unable to open device %s (%d)."),
			strUserDeviceName, GetLastError());
		AfxMessageBox(strErrorMsg, MB_OK);
	}
	else
	{
#endif
        if (m_nDevice == 0 || m_nDevice == 1)
        {
            // Set U1 port command byte to 0x80 (all 3 ports to Mode 0)
			byU1cmdvalue = 0x80;
        	IOWriteValue.dwPortNumber = static_cast<DWORD>(CMDPORT1);
	        IOWriteValue.byValue = static_cast<BYTE>(byU1cmdvalue);

#ifndef SOFTWARE_DEBUG
        	// Write port value.
	        if (!DeviceIoControl(m_ThreadData.hWIOHandle,
		        				 IOCTL_WRITE_BYTE,
			        			 &IOWriteValue,
				        		 sizeof(IO_STRUC),
					        	 NULL,
						         0,
    						     &dwActualOut,
	    					     NULL))
        	{
        		AfxMessageBox(_T("Error writing U1 command byte."), MB_OK);
				return;
	        }
			OnRadioaw1();
			OnRadiobw1();
			OnRadiocw1lo();
			OnRadiocw1hi();
#endif
        }
    
        if (m_nDevice == 0 || m_nDevice == 2)
        {
            // Set U2 port command byte to 0x80 (all 3 ports to Mode 0)
			byU2cmdvalue = 0x80;
        	IOWriteValue.dwPortNumber = static_cast<DWORD>(CMDPORT2);
	        IOWriteValue.byValue = static_cast<BYTE>(byU2cmdvalue);

#ifndef SOFTWARE_DEBUG
        	// Write port value
	        if (!DeviceIoControl(m_ThreadData.hWIOHandle,
		        				 IOCTL_WRITE_BYTE,
			        			 &IOWriteValue,
				        		 sizeof(IO_STRUC),
					        	 NULL,
						         0,
    						     &dwActualOut,
	    					    NULL))
    	    {
        		AfxMessageBox(_T("Error writing U2 command byte."), MB_OK);
				return;
            }
			OnRadioaw2();
			OnRadiobw2();
			OnRadiocw2lo();
			OnRadiocw2hi();
#endif
        }

        pwndCtrlNext = GetDlgItem(IDC_EDITAW1);
        pwndCtrlNext->SetFocus();

		m_byAR1Value = 0;       // zero the variables...
		m_byBR1Value = 0;
		m_byCR1LOValue = 0;
		m_byCR1HIValue = 0;
		m_byAW1Value = 0;
		m_byBW1Value = 0;
		m_byCW1LOValue = 0;
		m_byCW1HIValue = 0;
		m_byAR2Value = 0;
		m_byBR2Value = 0;
		m_byCR2LOValue = 0;
		m_byCR2HIValue = 0;
		m_byAW2Value = 0;
		m_byBW2Value = 0;
		m_byCW2LOValue = 0;
		m_byCW2HIValue = 0;
		UpdateData(FALSE);      // variable data to dialog now

		// Disable controls that are now invalid.
		m_ctlButtonOpenDevice.EnableWindow(FALSE);
		m_ctlDevice.EnableWindow(FALSE);

        // Enable additional controls that are now valid
		m_ctlButtonCloseDevice.EnableWindow(TRUE);

		deviceopen = TRUE;

		UpdateDlgControls(deviceopen);

#ifndef SOFTWARE_DEBUG
	}
#endif
}


void CIO48XPTestDlg::OnBUTTONCloseDriver() 
{
	if (deviceopen)
	{
#ifndef SOFTWARE_DEBUG
		// Signal the thread to exit
		m_ThreadData.fExitThread = TRUE;

		// Close the device driver
		if (INVALID_HANDLE_VALUE != m_ThreadData.hWIOHandle)
		{
			if (INVALID_HANDLE_VALUE != m_hDeviceThread)
			{
				// Signal the thread to exit
				SetEvent(m_ThreadData.hExitThread);

				// Wait for thread to exit
				WaitForSingleObject(m_ThreadData.hThreadExited, 3000);
				Sleep(10);

				// Close the thread handle
				CloseHandle(m_hDeviceThread);
				m_hDeviceThread = INVALID_HANDLE_VALUE;
			}

			// Close the device
			CloseHandle(m_ThreadData.hWIOHandle);
			m_ThreadData.hWIOHandle = INVALID_HANDLE_VALUE;
		}

		if (INVALID_HANDLE_VALUE == m_ThreadData.hExitThread)
		{
			CloseHandle(m_ThreadData.hExitThread);
			m_ThreadData.hExitThread = INVALID_HANDLE_VALUE;
		}

		if (INVALID_HANDLE_VALUE == m_ThreadData.hThreadExited)
		{
			CloseHandle(m_ThreadData.hThreadExited);
			m_ThreadData.hThreadExited = INVALID_HANDLE_VALUE;
		}
#endif

		m_byAR1Value = 0;       // zero the variables...
		m_byBR1Value = 0;
		m_byCR1LOValue = 0;
		m_byCR1HIValue = 0;
		m_byAW1Value = 0;
		m_byBW1Value = 0;
		m_byCW1LOValue = 0;
		m_byCW1HIValue = 0;
		m_byAR2Value = 0;
		m_byBR2Value = 0;
		m_byCR2LOValue = 0;
		m_byCR2HIValue = 0;
		m_byAW2Value = 0;
		m_byBW2Value = 0;
		m_byCW2LOValue = 0;
		m_byCW2HIValue = 0;
		UpdateData(FALSE);      // variable data to dialog now

	   	// Disable controls that are now invalid.
		m_ctlButtonCloseDevice.EnableWindow(FALSE);

	    // Enable additional controls that are now valid
		m_ctlButtonOpenDevice.EnableWindow(TRUE);
		m_ctlDevice.EnableWindow(TRUE);

		deviceopen = FALSE;

		UpdateDlgControls(deviceopen);
	}
}

void CIO48XPTestDlg::OnButtonar1() 
{
	BYTE	byPortValue = 0xFF;
	DWORD	dwPortNumber;
	DWORD	dwActualOut;

#ifndef SOFTWARE_DEBUG
	dwPortNumber = static_cast<DWORD>(PORTA1);
	// Read port value
	if (DeviceIoControl(m_ThreadData.hWIOHandle,
						IOCTL_READ_BYTE,
						&dwPortNumber,
						sizeof(DWORD),
						&byPortValue,
						sizeof(BYTE),
						&dwActualOut,
						NULL))
	{
        m_byAR1Value = static_cast<BYTE>(byPortValue);
        UpdateData(FALSE);      // variable data to dialog now
	}
	else
		AfxMessageBox(_T("Error reading U1 portA byte."), MB_OK);
#endif
}

void CIO48XPTestDlg::OnButtonbr1() 
{
	BYTE	byPortValue = 0xFF;
	DWORD	dwPortNumber;
	DWORD	dwActualOut;

#ifndef SOFTWARE_DEBUG
	dwPortNumber = static_cast<DWORD>(PORTB1);
	// Read port value
	if (DeviceIoControl(m_ThreadData.hWIOHandle,
						IOCTL_READ_BYTE,
						&dwPortNumber,
						sizeof(DWORD),
						&byPortValue,
						sizeof(BYTE),
						&dwActualOut,
						NULL))
	{
        m_byBR1Value = static_cast<BYTE>(byPortValue);
        UpdateData(FALSE);      // variable data to dialog now
	}
	else
		AfxMessageBox(_T("Error reading U1 portB byte."), MB_OK);
#endif
}

void CIO48XPTestDlg::OnButtoncr1lo() 
{
	BYTE	byPortValue = 0xFF;
	DWORD	dwPortNumber;
	DWORD	dwActualOut;

#ifndef SOFTWARE_DEBUG
	dwPortNumber = static_cast<DWORD>(PORTC1);
	// Read port value
	if (DeviceIoControl(m_ThreadData.hWIOHandle,
						IOCTL_READ_BYTE,
						&dwPortNumber,
						sizeof(DWORD),
						&byPortValue,
						sizeof(BYTE),
						&dwActualOut,
						NULL))
	{
		byC1value = static_cast<BYTE>(byPortValue);
		m_byCR1LOValue = byC1value & 0x0f;
        UpdateData(FALSE);      // variable data to dialog now
	}
	else
		AfxMessageBox(_T("Error reading U1 portC byte."), MB_OK);
#endif
}

void CIO48XPTestDlg::OnButtoncr1hi() 
{
	BYTE	byPortValue = 0xFF;
	DWORD	dwPortNumber;
	DWORD	dwActualOut;

#ifndef SOFTWARE_DEBUG
	dwPortNumber = static_cast<DWORD>(PORTC1);
	// Read port value
	if (DeviceIoControl(m_ThreadData.hWIOHandle,
						IOCTL_READ_BYTE,
						&dwPortNumber,
						sizeof(DWORD),
						&byPortValue,
						sizeof(BYTE),
						&dwActualOut,
						NULL))
	{
		byC1value = static_cast<BYTE>(byPortValue);
		m_byCR1HIValue = ((byC1value & 0xf0) >> 4);
        UpdateData(FALSE);      // variable data to dialog now
	}
	else
		AfxMessageBox(_T("Error reading U1 portC byte."), MB_OK);
#endif
}

void CIO48XPTestDlg::OnButtonar2() 
{
	BYTE	byPortValue = 0xFF;
	DWORD	dwPortNumber;
	DWORD	dwActualOut;

#ifndef SOFTWARE_DEBUG
	dwPortNumber = static_cast<DWORD>(PORTA2);
	// Read port value
	if (DeviceIoControl(m_ThreadData.hWIOHandle,
						IOCTL_READ_BYTE,
						&dwPortNumber,
						sizeof(DWORD),
						&byPortValue,
						sizeof(BYTE),
						&dwActualOut,
						NULL))
	{
        m_byAR2Value = static_cast<BYTE>(byPortValue);
        UpdateData(FALSE);      // variable data to dialog now
	}
	else
		AfxMessageBox(_T("Error reading U2 portA byte."), MB_OK);
#endif
}

void CIO48XPTestDlg::OnButtonbr2() 
{
	BYTE	byPortValue = 0xFF;
	DWORD	dwPortNumber;
	DWORD	dwActualOut;

#ifndef SOFTWARE_DEBUG
	dwPortNumber = static_cast<DWORD>(PORTB2);
	// Read port value
	if (DeviceIoControl(m_ThreadData.hWIOHandle,
						IOCTL_READ_BYTE,
						&dwPortNumber,
						sizeof(DWORD),
						&byPortValue,
						sizeof(BYTE),
						&dwActualOut,
						NULL))
	{
        m_byBR2Value = static_cast<BYTE>(byPortValue);
        UpdateData(FALSE);      // variable data to dialog now
	}
	else
		AfxMessageBox(_T("Error reading U2 portB byte."), MB_OK);
#endif
}


void CIO48XPTestDlg::OnButtoncr2lo() 
{
	BYTE	byPortValue = 0xFF;
	DWORD	dwPortNumber;
	DWORD	dwActualOut;

#ifndef SOFTWARE_DEBUG
	dwPortNumber = static_cast<DWORD>(PORTC2);
	// Read port value
	if (DeviceIoControl(m_ThreadData.hWIOHandle,
						IOCTL_READ_BYTE,
						&dwPortNumber,
						sizeof(DWORD),
						&byPortValue,
						sizeof(BYTE),
						&dwActualOut,
						NULL))
	{
		byC2value = static_cast<BYTE>(byPortValue);
		m_byCR2LOValue = byC2value & 0x0f;
        UpdateData(FALSE);      // variable data to dialog now
	}
	else
		AfxMessageBox(_T("Error reading U2 portC byte."), MB_OK);
#endif
}

void CIO48XPTestDlg::OnButtoncr2hi() 
{
	BYTE	byPortValue = 0xFF;
	DWORD	dwPortNumber;
	DWORD	dwActualOut;

#ifndef SOFTWARE_DEBUG
	dwPortNumber = static_cast<DWORD>(PORTC2);
	// Read port value
	if (DeviceIoControl(m_ThreadData.hWIOHandle,
						IOCTL_READ_BYTE,
						&dwPortNumber,
						sizeof(DWORD),
						&byPortValue,
						sizeof(BYTE),
						&dwActualOut,
						NULL))
	{
		byC2value = static_cast<BYTE>(byPortValue);
		m_byCR2HIValue = ((byC2value & 0xf0) >> 4);
        UpdateData(FALSE);      // variable data to dialog now
	}
	else
		AfxMessageBox(_T("Error reading U2 portC byte."), MB_OK);
#endif
}


void CIO48XPTestDlg::OnButtonaw1() 
{
	IO_STRUC	IOWriteValue;
	DWORD		dwActualOut;

	UpdateData(TRUE);           // dialog data to our variable

#ifndef SOFTWARE_DEBUG
	IOWriteValue.dwPortNumber = static_cast<DWORD>(PORTA1);
	IOWriteValue.byValue = static_cast<BYTE>(m_byAW1Value);

	// Write port value
	if (!DeviceIoControl(m_ThreadData.hWIOHandle,
                         IOCTL_WRITE_BYTE,
                         &IOWriteValue,
                         sizeof(IO_STRUC),
                         NULL,
                         0,
                         &dwActualOut,
                         NULL))
	{
		AfxMessageBox(_T("Error writing U1 portA byte."), MB_OK);
    }
#endif
}

void CIO48XPTestDlg::OnButtonbw1() 
{
	IO_STRUC	IOWriteValue;
	DWORD		dwActualOut;

	UpdateData(TRUE);           // dialog data to our variable

#ifndef SOFTWARE_DEBUG
	IOWriteValue.dwPortNumber = static_cast<DWORD>(PORTB1);
	IOWriteValue.byValue = static_cast<BYTE>(m_byBW1Value);

	// Write port value
	if (!DeviceIoControl(m_ThreadData.hWIOHandle,
                         IOCTL_WRITE_BYTE,
                         &IOWriteValue,
                         sizeof(IO_STRUC),
                         NULL,
                         0,
                         &dwActualOut,
                         NULL))
	{
		AfxMessageBox(_T("Error writing U1 portB byte."), MB_OK);
    }
#endif
}

void CIO48XPTestDlg::OnButtoncw1lo() 
{
	IO_STRUC	IOWriteValue;
	DWORD		dwActualOut;

	UpdateData(TRUE);           // dialog data to our variable

#ifndef SOFTWARE_DEBUG
	IOWriteValue.dwPortNumber = static_cast<DWORD>(PORTC1);
	byC1value &= 0xf0;
	byC1value |= (m_byCW1LOValue & 0x0f);
	IOWriteValue.byValue = static_cast<BYTE>(byC1value);

	// Write port value
	if (!DeviceIoControl(m_ThreadData.hWIOHandle,
						 IOCTL_WRITE_BYTE,
						 &IOWriteValue,
						 sizeof(IO_STRUC),
						 NULL,
						 0,
						 &dwActualOut,
						 NULL))
	{
		AfxMessageBox(_T("Error writing U1 portC byte."), MB_OK);
    }
#endif
}

void CIO48XPTestDlg::OnButtoncw1hi() 
{
	IO_STRUC	IOWriteValue;
	DWORD		dwActualOut;

	UpdateData(TRUE);           // dialog data to our variable

#ifndef SOFTWARE_DEBUG
	IOWriteValue.dwPortNumber = static_cast<DWORD>(PORTC1);
	byC1value &= 0x0f;
	byC1value |= (m_byCW1HIValue << 4);
	IOWriteValue.byValue = static_cast<BYTE>(byC1value);

	// Write port value
	if (!DeviceIoControl(m_ThreadData.hWIOHandle,
						 IOCTL_WRITE_BYTE,
						 &IOWriteValue,
						 sizeof(IO_STRUC),
						 NULL,
						 0,
						 &dwActualOut,
						 NULL))
	{
		AfxMessageBox(_T("Error writing U1 portC byte."), MB_OK);
    }
#endif
}

void CIO48XPTestDlg::OnButtonaw2() 
{
	IO_STRUC	IOWriteValue;
	DWORD		dwActualOut;

	UpdateData(TRUE);           // dialog data to our variable

#ifndef SOFTWARE_DEBUG
	IOWriteValue.dwPortNumber = static_cast<DWORD>(PORTA2);
	IOWriteValue.byValue = static_cast<BYTE>(m_byAW2Value);

	// Write port value
	if (!DeviceIoControl(m_ThreadData.hWIOHandle,
                         IOCTL_WRITE_BYTE,
                         &IOWriteValue,
                         sizeof(IO_STRUC),
                         NULL,
                         0,
                         &dwActualOut,
                         NULL))
	{
		AfxMessageBox(_T("Error writing U2 portA byte."), MB_OK);
    }
#endif
}

void CIO48XPTestDlg::OnButtonbw2() 
{
	IO_STRUC	IOWriteValue;
	DWORD		dwActualOut;

	UpdateData(TRUE);           // dialog data to our variable

#ifndef SOFTWARE_DEBUG
	IOWriteValue.dwPortNumber = static_cast<DWORD>(PORTB2);
	IOWriteValue.byValue = static_cast<BYTE>(m_byBW2Value);

	// Write port value
	if (!DeviceIoControl(m_ThreadData.hWIOHandle,
                         IOCTL_WRITE_BYTE,
                         &IOWriteValue,
                         sizeof(IO_STRUC),
                         NULL,
                         0,
                         &dwActualOut,
                         NULL))
	{
		AfxMessageBox(_T("Error writing U2 portB byte."), MB_OK);
    }
#endif
}


void CIO48XPTestDlg::OnButtoncw2lo() 
{
	IO_STRUC	IOWriteValue;
	DWORD		dwActualOut;

	UpdateData(TRUE);           // dialog data to our variable

#ifndef SOFTWARE_DEBUG
	IOWriteValue.dwPortNumber = static_cast<DWORD>(PORTC2);
	byC2value &= 0xf0;
	byC2value |= (m_byCW2LOValue & 0x0f);
	IOWriteValue.byValue = static_cast<BYTE>(byC2value);

	// Write port value
	if (!DeviceIoControl(m_ThreadData.hWIOHandle,
						 IOCTL_WRITE_BYTE,
						 &IOWriteValue,
						 sizeof(IO_STRUC),
						 NULL,
						 0,
						 &dwActualOut,
						 NULL))
	{
		AfxMessageBox(_T("Error writing U2 portC byte."), MB_OK);
    }
#endif
}

void CIO48XPTestDlg::OnButtoncw2hi() 
{
	IO_STRUC	IOWriteValue;
	DWORD		dwActualOut;

	UpdateData(TRUE);           // dialog data to our variable

#ifndef SOFTWARE_DEBUG
	IOWriteValue.dwPortNumber = static_cast<DWORD>(PORTC2);
	byC2value &= 0x0f;
	byC2value |= (m_byCW2HIValue << 4);
	IOWriteValue.byValue = static_cast<BYTE>(byC2value);

	// Write port value
	if (!DeviceIoControl(m_ThreadData.hWIOHandle,
						 IOCTL_WRITE_BYTE,
						 &IOWriteValue,
						 sizeof(IO_STRUC),
						 NULL,
						 0,
						 &dwActualOut,
						 NULL))
	{
		AfxMessageBox(_T("Error writing U2 portC byte."), MB_OK);
    }
#endif
}


void CIO48XPTestDlg::OnRadioar1() 
{
	IO_STRUC	IOWriteValue;
	DWORD		dwActualOut;

	UpdateData(TRUE);           // dialog data to our variable

#ifndef SOFTWARE_DEBUG
	byU1cmdvalue |= 0x10;			// set B4 to setup portA as inputs
   	IOWriteValue.dwPortNumber = static_cast<DWORD>(CMDPORT1);
	IOWriteValue.byValue = static_cast<BYTE>(byU1cmdvalue);

	m_byAW1Value = 0;

    UpdateData(FALSE);      // variable data to dialog now

   	// Write command port value.
	if (!DeviceIoControl(m_ThreadData.hWIOHandle,
        				 IOCTL_WRITE_BYTE,
	        			 &IOWriteValue,
		        		 sizeof(IO_STRUC),
			        	 NULL,
				         0,
					     &dwActualOut,
   					     NULL))
   	{
   		AfxMessageBox(_T("Error writing U1 command byte."), MB_OK);
	}
#endif

    if (m_nDevice == 0 || m_nDevice == 1)
    {
        m_ctlButtonAW1.EnableWindow(FALSE);
        m_AW1EditCtrl.EnableWindow(FALSE);
	}
}

void CIO48XPTestDlg::OnRadiobr1() 
{
	IO_STRUC	IOWriteValue;
	DWORD		dwActualOut;

	UpdateData(TRUE);           // dialog data to our variable

#ifndef SOFTWARE_DEBUG
	byU1cmdvalue |= 0x02;			// set B1 to setup portB as inputs
   	IOWriteValue.dwPortNumber = static_cast<DWORD>(CMDPORT1);
	IOWriteValue.byValue = static_cast<BYTE>(byU1cmdvalue);

	m_byBW1Value = 0;

    UpdateData(FALSE);      // variable data to dialog now

   	// Write command port value.
	if (!DeviceIoControl(m_ThreadData.hWIOHandle,
        				 IOCTL_WRITE_BYTE,
	        			 &IOWriteValue,
		        		 sizeof(IO_STRUC),
			        	 NULL,
				         0,
					     &dwActualOut,
   					     NULL))
   	{
   		AfxMessageBox(_T("Error writing U1 command byte."), MB_OK);
	}
#endif

    if (m_nDevice == 0 || m_nDevice == 1)
    {
        m_ctlButtonBW1.EnableWindow(FALSE);
        m_BW1EditCtrl.EnableWindow(FALSE);
	}
}

void CIO48XPTestDlg::OnRadiocr1lo() 
{
	IO_STRUC	IOWriteValue;
	DWORD		dwActualOut;

	UpdateData(TRUE);           // dialog data to our variable

#ifndef SOFTWARE_DEBUG
	byU1cmdvalue |= 0x01;			// set B0 to setup portC low nibble as inputs
   	IOWriteValue.dwPortNumber = static_cast<DWORD>(CMDPORT1);
	IOWriteValue.byValue = static_cast<BYTE>(byU1cmdvalue);

	m_byCW1LOValue = 0;

    UpdateData(FALSE);      // variable data to dialog now

   	// Write command port value.
	if (!DeviceIoControl(m_ThreadData.hWIOHandle,
        				 IOCTL_WRITE_BYTE,
	        			 &IOWriteValue,
		        		 sizeof(IO_STRUC),
			        	 NULL,
				         0,
					     &dwActualOut,
   					     NULL))
   	{
   		AfxMessageBox(_T("Error writing U1 command byte."), MB_OK);
	}
#endif

    if (m_nDevice == 0 || m_nDevice == 1)
    {
        m_ctlButtonCW1LO.EnableWindow(FALSE);
        m_CW1LOEditCtrl.EnableWindow(FALSE);
	}
}


void CIO48XPTestDlg::OnRadiocr1hi() 
{
	IO_STRUC	IOWriteValue;
	DWORD		dwActualOut;

	UpdateData(TRUE);           // dialog data to our variable

#ifndef SOFTWARE_DEBUG
	byU1cmdvalue |= 0x08;			// set B3 to setup portC high nibble as inputs
   	IOWriteValue.dwPortNumber = static_cast<DWORD>(CMDPORT1);
	IOWriteValue.byValue = static_cast<BYTE>(byU1cmdvalue);

	m_byCW1HIValue = 0;

    UpdateData(FALSE);      // variable data to dialog now

   	// Write command port value.
	if (!DeviceIoControl(m_ThreadData.hWIOHandle,
        				 IOCTL_WRITE_BYTE,
	        			 &IOWriteValue,
		        		 sizeof(IO_STRUC),
			        	 NULL,
				         0,
					     &dwActualOut,
   					     NULL))
   	{
   		AfxMessageBox(_T("Error writing U1 command byte."), MB_OK);
	}
#endif

    if (m_nDevice == 0 || m_nDevice == 1)
    {
        m_ctlButtonCW1HI.EnableWindow(FALSE);
        m_CW1HIEditCtrl.EnableWindow(FALSE);
	}
}

void CIO48XPTestDlg::OnRadioar2() 
{
	IO_STRUC	IOWriteValue;
	DWORD		dwActualOut;

	UpdateData(TRUE);           // dialog data to our variable

#ifndef SOFTWARE_DEBUG
	byU2cmdvalue |= 0x10;			// set B4 to setup portA as inputs
   	IOWriteValue.dwPortNumber = static_cast<DWORD>(CMDPORT2);
	IOWriteValue.byValue = static_cast<BYTE>(byU2cmdvalue);

	m_byAW2Value = 0;

    UpdateData(FALSE);      // variable data to dialog now

   	// Write command port value.
	if (!DeviceIoControl(m_ThreadData.hWIOHandle,
        				 IOCTL_WRITE_BYTE,
	        			 &IOWriteValue,
		        		 sizeof(IO_STRUC),
			        	 NULL,
				         0,
					     &dwActualOut,
   					     NULL))
   	{
   		AfxMessageBox(_T("Error writing U2 command byte."), MB_OK);
	}
#endif

    if (m_nDevice == 0 || m_nDevice == 2)
    {
        m_ctlButtonAW2.EnableWindow(FALSE);
        m_AW2EditCtrl.EnableWindow(FALSE);
	}
}

void CIO48XPTestDlg::OnRadiobr2() 
{
	IO_STRUC	IOWriteValue;
	DWORD		dwActualOut;

	UpdateData(TRUE);           // dialog data to our variable

#ifndef SOFTWARE_DEBUG
	byU2cmdvalue |= 0x02;			// set B1 to setup portB as inputs
   	IOWriteValue.dwPortNumber = static_cast<DWORD>(CMDPORT2);
	IOWriteValue.byValue = static_cast<BYTE>(byU2cmdvalue);

	m_byBW2Value = 0;

    UpdateData(FALSE);      // variable data to dialog now

   	// Write command port value.
	if (!DeviceIoControl(m_ThreadData.hWIOHandle,
        				 IOCTL_WRITE_BYTE,
	        			 &IOWriteValue,
		        		 sizeof(IO_STRUC),
			        	 NULL,
				         0,
					     &dwActualOut,
   					     NULL))
   	{
   		AfxMessageBox(_T("Error writing U2 command byte."), MB_OK);
	}
#endif

    if (m_nDevice == 0 || m_nDevice == 2)
    {
        m_ctlButtonBW2.EnableWindow(FALSE);
        m_BW2EditCtrl.EnableWindow(FALSE);
	}
}

void CIO48XPTestDlg::OnRadiocr2lo() 
{
	IO_STRUC	IOWriteValue;
	DWORD		dwActualOut;

	UpdateData(TRUE);           // dialog data to our variable

#ifndef SOFTWARE_DEBUG
	byU2cmdvalue |= 0x01;			// set B0 to setup portC low nibble as inputs
   	IOWriteValue.dwPortNumber = static_cast<DWORD>(CMDPORT2);
	IOWriteValue.byValue = static_cast<BYTE>(byU2cmdvalue);

	m_byCW2LOValue = 0;

    UpdateData(FALSE);      // variable data to dialog now

   	// Write command port value.
	if (!DeviceIoControl(m_ThreadData.hWIOHandle,
        				 IOCTL_WRITE_BYTE,
	        			 &IOWriteValue,
		        		 sizeof(IO_STRUC),
			        	 NULL,
				         0,
					     &dwActualOut,
   					     NULL))
   	{
   		AfxMessageBox(_T("Error writing U2 command byte."), MB_OK);
	}
#endif

    if (m_nDevice == 0 || m_nDevice == 2)
    {
        m_ctlButtonCW2LO.EnableWindow(FALSE);
        m_CW2LOEditCtrl.EnableWindow(FALSE);
	}
}

void CIO48XPTestDlg::OnRadiocr2hi() 
{
	IO_STRUC	IOWriteValue;
	DWORD		dwActualOut;

	UpdateData(TRUE);           // dialog data to our variable

#ifndef SOFTWARE_DEBUG
	byU2cmdvalue |= 0x08;			// set B3 to setup portC high nibble as inputs
   	IOWriteValue.dwPortNumber = static_cast<DWORD>(CMDPORT2);
	IOWriteValue.byValue = static_cast<BYTE>(byU2cmdvalue);

	m_byCW2HIValue = 0;

    UpdateData(FALSE);      // variable data to dialog now

   	// Write command port value.
	if (!DeviceIoControl(m_ThreadData.hWIOHandle,
        				 IOCTL_WRITE_BYTE,
	        			 &IOWriteValue,
		        		 sizeof(IO_STRUC),
			        	 NULL,
				         0,
					     &dwActualOut,
   					     NULL))
   	{
   		AfxMessageBox(_T("Error writing U2 command byte."), MB_OK);
	}
#endif

    if (m_nDevice == 0 || m_nDevice == 2)
    {
        m_ctlButtonCW2HI.EnableWindow(FALSE);
        m_CW2HIEditCtrl.EnableWindow(FALSE);
	}
}

void CIO48XPTestDlg::OnRadioaw1() 
{
	IO_STRUC	IOWriteValue;
	DWORD		dwActualOut;

	UpdateData(TRUE);           // dialog data to our variable

#ifndef SOFTWARE_DEBUG
	byU1cmdvalue &= 0xEF;			// clear B4 to setup portA as outputs
   	IOWriteValue.dwPortNumber = static_cast<DWORD>(CMDPORT1);
	IOWriteValue.byValue = static_cast<BYTE>(byU1cmdvalue);

   	// Write command port value.
	if (!DeviceIoControl(m_ThreadData.hWIOHandle,
        				 IOCTL_WRITE_BYTE,
	        			 &IOWriteValue,
		        		 sizeof(IO_STRUC),
			        	 NULL,
				         0,
					     &dwActualOut,
   					     NULL))
   	{
   		AfxMessageBox(_T("Error writing U1 command byte."), MB_OK);
	}
#endif

    if (m_nDevice == 0 || m_nDevice == 1)
    {
        m_ctlButtonAW1.EnableWindow(TRUE);
        m_AW1EditCtrl.EnableWindow(TRUE);
	}
}

void CIO48XPTestDlg::OnRadiobw1() 
{
	IO_STRUC	IOWriteValue;
	DWORD		dwActualOut;

	UpdateData(TRUE);           // dialog data to our variable

#ifndef SOFTWARE_DEBUG
	byU1cmdvalue &= 0xFD;			// clear B1 to setup portB as outputs
   	IOWriteValue.dwPortNumber = static_cast<DWORD>(CMDPORT1);
	IOWriteValue.byValue = static_cast<BYTE>(byU1cmdvalue);

   	// Write command port value.
	if (!DeviceIoControl(m_ThreadData.hWIOHandle,
        				 IOCTL_WRITE_BYTE,
	        			 &IOWriteValue,
		        		 sizeof(IO_STRUC),
			        	 NULL,
				         0,
					     &dwActualOut,
   					     NULL))
   	{
   		AfxMessageBox(_T("Error writing U1 command byte."), MB_OK);
	}
#endif

    if (m_nDevice == 0 || m_nDevice == 1)
    {
        m_ctlButtonBW1.EnableWindow(TRUE);
        m_BW1EditCtrl.EnableWindow(TRUE);
	}
}

void CIO48XPTestDlg::OnRadiocw1lo() 
{
	IO_STRUC	IOWriteValue;
	DWORD		dwActualOut;

	UpdateData(TRUE);           // dialog data to our variable

#ifndef SOFTWARE_DEBUG
	byU1cmdvalue &= 0xFE;			// clear B0 to setup portC low nibble as outputs
   	IOWriteValue.dwPortNumber = static_cast<DWORD>(CMDPORT1);
	IOWriteValue.byValue = static_cast<BYTE>(byU1cmdvalue);

   	// Write command port value.
	if (!DeviceIoControl(m_ThreadData.hWIOHandle,
        				 IOCTL_WRITE_BYTE,
	        			 &IOWriteValue,
		        		 sizeof(IO_STRUC),
			        	 NULL,
				         0,
					     &dwActualOut,
   					     NULL))
   	{
   		AfxMessageBox(_T("Error writing U1 command byte."), MB_OK);
	}
#endif

    if (m_nDevice == 0 || m_nDevice == 1)
    {
        m_ctlButtonCW1LO.EnableWindow(TRUE);
        m_CW1LOEditCtrl.EnableWindow(TRUE);
	}
}

void CIO48XPTestDlg::OnRadiocw1hi() 
{
	IO_STRUC	IOWriteValue;
	DWORD		dwActualOut;

	UpdateData(TRUE);           // dialog data to our variable

#ifndef SOFTWARE_DEBUG
	byU1cmdvalue &= 0xF7;			// clear B3 to setup portC high nibble as outputs
   	IOWriteValue.dwPortNumber = static_cast<DWORD>(CMDPORT1);
	IOWriteValue.byValue = static_cast<BYTE>(byU1cmdvalue);

   	// Write command port value.
	if (!DeviceIoControl(m_ThreadData.hWIOHandle,
        				 IOCTL_WRITE_BYTE,
	        			 &IOWriteValue,
		        		 sizeof(IO_STRUC),
			        	 NULL,
				         0,
					     &dwActualOut,
   					     NULL))
   	{
   		AfxMessageBox(_T("Error writing U1 command byte."), MB_OK);
	}
#endif

    if (m_nDevice == 0 || m_nDevice == 1)
    {
        m_ctlButtonCW1HI.EnableWindow(TRUE);
        m_CW1HIEditCtrl.EnableWindow(TRUE);
	}
}

void CIO48XPTestDlg::OnRadioaw2() 
{
	IO_STRUC	IOWriteValue;
	DWORD		dwActualOut;

	UpdateData(TRUE);           // dialog data to our variable

#ifndef SOFTWARE_DEBUG
	byU2cmdvalue &= 0xEF;			// clear B4 to setup portA as outputs
   	IOWriteValue.dwPortNumber = static_cast<DWORD>(CMDPORT2);
	IOWriteValue.byValue = static_cast<BYTE>(byU2cmdvalue);

   	// Write command port value.
	if (!DeviceIoControl(m_ThreadData.hWIOHandle,
        				 IOCTL_WRITE_BYTE,
	        			 &IOWriteValue,
		        		 sizeof(IO_STRUC),
			        	 NULL,
				         0,
					     &dwActualOut,
   					     NULL))
   	{
   		AfxMessageBox(_T("Error writing U2 command byte."), MB_OK);
	}
#endif

    if (m_nDevice == 0 || m_nDevice == 2)
    {
        m_ctlButtonAW2.EnableWindow(TRUE);
        m_AW2EditCtrl.EnableWindow(TRUE);
	}
}

void CIO48XPTestDlg::OnRadiobw2() 
{
	IO_STRUC	IOWriteValue;
	DWORD		dwActualOut;

	UpdateData(TRUE);           // dialog data to our variable

#ifndef SOFTWARE_DEBUG
	byU2cmdvalue &= 0xFD;			// clear B1 to setup portB as outputs
   	IOWriteValue.dwPortNumber = static_cast<DWORD>(CMDPORT2);
	IOWriteValue.byValue = static_cast<BYTE>(byU2cmdvalue);

   	// Write command port value.
	if (!DeviceIoControl(m_ThreadData.hWIOHandle,
        				 IOCTL_WRITE_BYTE,
	        			 &IOWriteValue,
		        		 sizeof(IO_STRUC),
			        	 NULL,
				         0,
					     &dwActualOut,
   					     NULL))
   	{
   		AfxMessageBox(_T("Error writing U2 command byte."), MB_OK);
	}
#endif

    if (m_nDevice == 0 || m_nDevice == 2)
    {
        m_ctlButtonBW2.EnableWindow(TRUE);
        m_BW2EditCtrl.EnableWindow(TRUE);
	}
}

void CIO48XPTestDlg::OnRadiocw2lo() 
{
	IO_STRUC	IOWriteValue;
	DWORD		dwActualOut;

	UpdateData(TRUE);           // dialog data to our variable

#ifndef SOFTWARE_DEBUG
	byU2cmdvalue &= 0xFE;			// clear B0 to setup portC low nibble as outputs
   	IOWriteValue.dwPortNumber = static_cast<DWORD>(CMDPORT2);
	IOWriteValue.byValue = static_cast<BYTE>(byU2cmdvalue);

   	// Write command port value.
	if (!DeviceIoControl(m_ThreadData.hWIOHandle,
        				 IOCTL_WRITE_BYTE,
	        			 &IOWriteValue,
		        		 sizeof(IO_STRUC),
			        	 NULL,
				         0,
					     &dwActualOut,
   					     NULL))
   	{
   		AfxMessageBox(_T("Error writing U2 command byte."), MB_OK);
	}
#endif

    if (m_nDevice == 0 || m_nDevice == 2)
    {
        m_ctlButtonCW2LO.EnableWindow(TRUE);
        m_CW2LOEditCtrl.EnableWindow(TRUE);
	}
}


void CIO48XPTestDlg::OnRadiocw2hi() 
{
	IO_STRUC	IOWriteValue;
	DWORD		dwActualOut;

	UpdateData(TRUE);           // dialog data to our variable

#ifndef SOFTWARE_DEBUG
	byU2cmdvalue &= 0xF7;			// clear B3 to setup portC high nibble as outputs
   	IOWriteValue.dwPortNumber = static_cast<DWORD>(CMDPORT2);
	IOWriteValue.byValue = static_cast<BYTE>(byU2cmdvalue);

   	// Write command port value.
	if (!DeviceIoControl(m_ThreadData.hWIOHandle,
        				 IOCTL_WRITE_BYTE,
	        			 &IOWriteValue,
		        		 sizeof(IO_STRUC),
			        	 NULL,
				         0,
					     &dwActualOut,
   					     NULL))
   	{
   		AfxMessageBox(_T("Error writing U2 command byte."), MB_OK);
	}
#endif

    if (m_nDevice == 0 || m_nDevice == 2)
    {
        m_ctlButtonCW2HI.EnableWindow(TRUE);
        m_CW2HIEditCtrl.EnableWindow(TRUE);
	}
}


