//---------------------------------------------------------------------------------------------------------------------------------
// Module/file name: IO48XPTestDlg.h
// Module Description: IO48XPTestDlg class header file
// Creation date: 11/17/07
// Author/s: George Hennessee
// Executable program name: IO48XPTest.exe
// Executable program type: Windows 2K/XP Application
//
// NOTE: This source code is written assuming a 132-character line width. For decent readability select font, page width, page
//       orientation, etc. to yield a 132-character unbroken line on your word processor screen. Be aware that any saving of this
//       source code file must be in TEXT ONLY format (no embedded codes) to avoid choking the C compiler. - GH
// Revision History:
//    Date      Release Rev    Author    Description
//  --------    -----------    ------    --------------------------------------------------------------------------------
//  11/19/07       1.00          GH      Original release
//
//              ******************************** COPYRIGHT NOTICE AND DISCLAIMER ********************************
//   This source file is Copyright 2007 by WinSystems, Inc., Arlington, Texas. All rights reserved. This file is provided 'as-is'
//   and WinSystems makes no warranty as to usability of this code. Disclosure or use of this code except as provided below is
//   strictly prohibited. Refer to the WinSystems Hardware/Software Sales Agreement for other terms and conditions that may apply.
//---------------------------------------------------------------------------------------------------------------------------------

#if !defined(AFX_IO48XPTESTDLG_H__A6909327_8C58_414C_A418_688B29B78CA8__INCLUDED_)
#define AFX_IO48XPTESTDLG_H__A6909327_8C58_414C_A418_688B29B78CA8__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


#include <winioctl.h>
#include "..\IO48XPTest\Ioctls.h"

typedef struct _ThreadData
{
	// Flag to indicate the thread should exit.
	BOOL	fExitThread;

	// Handle to UIO device.
	HANDLE	hWIOHandle;

	// Handle to event to signal thread to exit.
	HANDLE	hExitThread;

	// Handle to event to signal thread exited.
	HANDLE	hThreadExited;

	// Handle of dialog window.
	HWND	hwndDialog;

	// Critical section to govern data access to the thread data.
	CRITICAL_SECTION	hThreadCS;

} THREAD_DATA, *PTHREAD_DATA;


/////////////////////////////////////////////////////////////////////////////
// CIO48XPTestDlg dialog

class CIO48XPTestDlg : public CDialog
{
	// Handle to thread to handle events from the driver.
	HANDLE	m_hDeviceThread;

	// Data to share with the thread to handle events from the driver.
	THREAD_DATA		m_ThreadData;

// Construction
public:
	CIO48XPTestDlg(CWnd* pParent = NULL);	// standard constructor

// Dialog Data
	//{{AFX_DATA(CIO48XPTestDlg)
	enum { IDD = IDD_IO48XPTEST_DIALOG };
	CEdit	m_CW2LOEditCtrl;
	CEdit	m_CW2HIEditCtrl;
	CEdit	m_CR2LOEditCtrl;
	CEdit	m_CR2HIEditCtrl;
	CButton	m_ctlButtonCW2LO;
	CButton	m_ctlButtonCW2HI;
	CButton	m_ctlButtonCR2LO;
	CButton	m_ctlButtonCR2HI;
	CEdit	m_CW1HIEditCtrl;
	CEdit	m_CR1HIEditCtrl;
	CButton	m_ctlButtonCW1HI;
	CButton	m_ctlButtonCR1HI;
	CEdit	m_CW1LOEditCtrl;
	CEdit	m_CR1LOEditCtrl;
	CButton	m_ctlButtonCW1LO;
	CButton	m_ctlButtonCR1LO;
	CButton	m_ctlButtonBW2;
	CButton	m_ctlButtonBW1;
	CButton	m_ctlButtonBR2;
	CButton	m_ctlButtonBR1;
	CButton	m_ctlButtonAW2;
	CButton	m_ctlButtonAW1;
	CButton	m_ctlButtonAR2;
	CButton	m_ctlButtonAR1;
	CButton	m_ctlButtonCloseDevice;
	CButton	m_ctlButtonOpenDevice;
	CComboBox	m_ctlDevice;
	CEdit	m_BW2EditCtrl;
	CEdit	m_BW1EditCtrl;
	CEdit	m_BR2EditCtrl;
	CEdit	m_BR1EditCtrl;
	CEdit	m_AW2EditCtrl;
	CEdit	m_AW1EditCtrl;
	CEdit	m_AR2EditCtrl;
	CEdit	m_AR1EditCtrl;
	BYTE	m_byAR1Value;
	BYTE	m_byAR2Value;
	BYTE	m_byAW1Value;
	BYTE	m_byAW2Value;
	BYTE	m_byBR1Value;
	BYTE	m_byBR2Value;
	BYTE	m_byBW1Value;
	BYTE	m_byBW2Value;
	int		m_nDevice;
	int		m_nModeA1;
	int		m_nModeB1;
	BYTE	m_byCR1LOValue;
	BYTE	m_byCW1LOValue;
	int		m_nModeC1LO;
	BYTE	m_byCR1HIValue;
	BYTE	m_byCW1HIValue;
	int		m_nModeC1HI;
	BYTE	m_byCR2HIValue;
	BYTE	m_byCR2LOValue;
	BYTE	m_byCW2HIValue;
	BYTE	m_byCW2LOValue;
	int		m_nModeA2;
	int		m_nModeB2;
	int		m_nModeC2HI;
	int		m_nModeC2LO;
	//}}AFX_DATA

	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CIO48XPTestDlg)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);	// DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
protected:
	HICON m_hIcon;

	BOOL	deviceopen;
	BYTE	byU1cmdvalue;
	BYTE	byU2cmdvalue;
	BYTE    byC1value;
	BYTE	byC2value;

	static DWORD WINAPI DeviceThread(PVOID pContext);

	void	UpdateDlgControls(BOOL bena);

    virtual void OnOK();

	// Generated message map functions
	//{{AFX_MSG(CIO48XPTestDlg)
	virtual BOOL OnInitDialog();
	afx_msg void OnSysCommand(UINT nID, LPARAM lParam);
	afx_msg void OnPaint();
	afx_msg HCURSOR OnQueryDragIcon();
	afx_msg void OnBUTTONCloseDriver();
	afx_msg void OnButtoncmd1();
	afx_msg void OnButtoncmd2();
	afx_msg void OnBUTTONOpenDriver();
	afx_msg void OnDestroy();
	afx_msg void OnButtonar1();
	afx_msg void OnButtonar2();
	afx_msg void OnButtonbr1();
	afx_msg void OnButtonbr2();
	afx_msg void OnButtonbw1();
	afx_msg void OnButtonbw2();
	afx_msg void OnButtonaw1();
	afx_msg void OnButtonaw2();
	afx_msg void OnClickedOk();
	afx_msg void OnSelchangeCOMBODeviceName();
	afx_msg void OnRadioaw1();
	afx_msg void OnRadioar1();
	afx_msg void OnRadiobr1();
	afx_msg void OnRadiobw1();
	afx_msg void OnButtoncr1lo();
	afx_msg void OnButtoncw1lo();
	afx_msg void OnRadiocr1lo();
	afx_msg void OnRadiocw1lo();
	afx_msg void OnRadiocr1hi();
	afx_msg void OnRadiocw1hi();
	afx_msg void OnButtoncw1hi();
	afx_msg void OnButtoncr1hi();
	afx_msg void OnButtoncr2hi();
	afx_msg void OnButtoncr2lo();
	afx_msg void OnButtoncw2hi();
	afx_msg void OnButtoncw2lo();
	afx_msg void OnRadioar2();
	afx_msg void OnRadioaw2();
	afx_msg void OnRadiobr2();
	afx_msg void OnRadiobw2();
	afx_msg void OnRadiocr2hi();
	afx_msg void OnRadiocr2lo();
	afx_msg void OnRadiocw2hi();
	afx_msg void OnRadiocw2lo();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_IO48XPTESTDLG_H__A6909327_8C58_414C_A418_688B29B78CA8__INCLUDED_)
