//---------------------------------------------------------------------------------------------------------------------------------
// Module/file name: IOCTLS.h
// Module Description: IOCTL code definitions for the IO48XP driver
// Creation date: 11/17/07
// Author/s: George Hennessee
// Executable program name: IO48XP.sys
// Executable program type: Windows WDM device driver
//
// NOTE: This source code is written assuming a 132-character line width. For decent readability select font, page width, page
//       orientation, etc. to yield a 132-character unbroken line on your word processor screen. Be aware that any saving of this
//       source code file must be in TEXT ONLY format (no embedded codes) to avoid choking the C compiler. - GH
// Revision History:
//    Date      Release Rev    Author    Description
//  --------    -----------    ------    --------------------------------------------------------------------------------
//  11/19/07       1.00          GH      Original release
//
//              ******************************** COPYRIGHT NOTICE AND DISCLAIMER ********************************
//   This source file is Copyright 2007 by WinSystems, Inc., Arlington, Texas. All rights reserved. This file is provided 'as-is'
//   and WinSystems makes no warranty as to usability of this code. Disclosure or use of this code except as provided below is
//   strictly prohibited. Refer to the WinSystems Hardware/Software Sales Agreement for other terms and conditions that may apply.
//---------------------------------------------------------------------------------------------------------------------------------

#ifndef IOCTLS_H
    #define IOCTLS_H

#ifndef CTL_CODE
    #pragma message("CTL_CODE undefined. Include winioctl.h or wdm.h")
#endif

// IO48 Register Definitions (driver will decrement this value by 1 to obtain the
// port address offset from base address
#define PORTA1         1	// Chip 1, Port A
#define PORTB1         2	// Chip 1, Port B
#define PORTC1         3	// Chip 1, Port C
#define CMDPORT1       4	// Chip 1, Command Port
#define PORTA2         5	// Chip 2, Port A
#define PORTB2         6	// Chip 2, Port B
#define PORTC2         7	// Chip 2, Port C
#define CMDPORT2       8	// Chip 2, Command Port

// Maximum number of IO ports
#define MAX_IO_PORTS         8

typedef unsigned char BYTE;
typedef unsigned char* PBYTE;
typedef unsigned long DWORD;
typedef unsigned long* PDWORD;

// Structure to use when writing a value to an I/O point or port using IOCTL_WIO_WRITE_BIT and IOCTL_WIO_WRITE_BYTE
typedef struct _IOStruct
{
    DWORD  dwPortNumber;                                   // Port number to access - valid range: 1-3 (per chip)
    BYTE   byValue;                                        // Value to write, 1 or 0
} IO_STRUC, *PIO_STRUC;


// Define the type of device. This parameter can be no bigger than a WORD value. The values used by Microsoft are in the range of
// 0-32767 and the values between 32768-65535 are reserved for use by OEMs and IHVs
#define IO_DEV_TYPE         32768


//-------------------------------------------------------------------------------------------------------------------------
// IOCTL_READ_BYTE - Device IO control routine to get the state of all I/O pins in the specified port
// dwCode          IOCTL_READ_BYTE
// dwInLen         [IN] Size of input data buffer, should be sizeof(DWORD)
// dwOutLen        [IN] Size of output data buffer, should be sizeof(BYTE)
// pSysBuf         [IN/OUT]
//                 [IN] Pointer to a DWORD containing the port number to read - valid range: 1-3 (per chip)
//                 [OUT] Pointer to a BYTE containing the state of the I/O port
// dwRetLen        [OUT] Number of output bytes put into the copy buffer
//-------------------------------------------------------------------------------------------------------------------------
#define IOCTL_READ_BYTE   CTL_CODE(IO_DEV_TYPE, 0x804, METHOD_BUFFERED, FILE_ANY_ACCESS)

//-------------------------------------------------------------------------------------------------------------------------
// IOCTL_WRITE_BYTE - Device IO control routine to set the state of all I/O pins in the specified port
// dwCode          IOCTL_WRITE_BYTE
// dwInLen         [IN] Size of input data buffer, should be sizeof(IO_STRUC)
// dwOutLen        Ignored
// pSysBuf         [IN] Pointer to a IO_STRUC structure
//                           IO_STRUC structure
//                           byPortNum      The port number to write value to - valid range: 1-3 (per chip)
//                           byValue        Value to write to port
// dwRetLen        [OUT] Number of output bytes put into the copy buffer
//-------------------------------------------------------------------------------------------------------------------------
#define IOCTL_WRITE_BYTE   CTL_CODE(IO_DEV_TYPE, 0x805, METHOD_BUFFERED, FILE_ANY_ACCESS)


#endif

